Package: MIMSunit
Type: Package
Title: Algorithm to Compute Monitor Independent Movement Summary Unit
        (MIMS-Unit)
Version: 0.11.3
Date: 2026-05-13
Authors@R: c(person("Qu", "Tang", email = "tang.q@northeastern.edu",
                  role = c("aut"), comment = c(ORCID = "0000-0001-5415-0205")),
             person("Dinesh", "John", email = "d.john@northeastern.edu",
                  role = c("aut")),
             person("Stephen", "Intille", email = "s.intille@northeastern.edu",
                  role = c("aut")),
             person("Umberto", "Mazzucchelli", email = "mazzucchelli.u@northeastern.edu",
                  role = c("cre"), comment = c(ORCID = "0009-0000-1071-6027")),
             person("mHealth Research Group", role = c("cph"), 
             comment = "https://www.mhealthgroup.org")
            )
Description: The MIMS-unit algorithm is developed to compute Monitor Independent
    Movement Summary Unit, a measurement to summarize raw accelerometer data 
    while ensuring harmonized results across different devices. It also includes
    scripts to reproduce results in the related publication 
    (John, D., Tang. Q., Albinali, F. and Intille, S. (2019) <doi:10.1123/jmpb.2018-0068>).
License: MIT + file LICENSE
Language: en-US
Depends: R (>= 3.6.0)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.0
URL: https://mhealthgroup.github.io/MIMSunit/,
        https://github.com/mhealthgroup/MIMSunit/tree/master
BugReports: https://github.com/mhealthgroup/MIMSunit/issues/
Imports: caTools (>= 1.17.1.1), tibble (>= 3.0.4), dplyr (>= 0.7.7),
        lubridate (>= 1.7.4), magrittr (>= 1.5), plyr (>= 1.8.4), readr
        (>= 1.1.1), R.utils (>= 2.7.0), stringr (>= 1.3.1), xts (>=
        0.11-2), signal (>= 0.7-7), dygraphs (>= 1.1.1.6), shiny (>=
        1.4.0.2), RColorBrewer (>= 1.1-2), utils (>= 3.6.1), ggplot2
        (>= 3.2.1)
Suggests: testthat, pkgdown, gridExtra, remotes
SystemRequirements: memory (>= 4GB) Ubuntu: build-essential,
        libxml2-dev, libssl-dev, libcurl4-openssl-dev Windows: Rtools
        (>= 3.5)
NeedsCompilation: no
Packaged: 2026-05-13 21:00:09 UTC; umberto
Author: Qu Tang [aut] (ORCID: <https://orcid.org/0000-0001-5415-0205>),
  Dinesh John [aut],
  Stephen Intille [aut],
  Umberto Mazzucchelli [cre] (ORCID:
    <https://orcid.org/0009-0000-1071-6027>),
  mHealth Research Group [cph] (https://www.mhealthgroup.org)
Maintainer: Umberto Mazzucchelli <mazzucchelli.u@northeastern.edu>
Repository: CRAN
Date/Publication: 2026-05-14 14:00:11 UTC
Built: R 4.5.2; ; 2026-05-14 14:22:03 UTC; unix
