# Generated by roxygen2: do not edit by hand

export(aggregate_for_mims)
export(aggregate_for_orientation)
export(bandlimited_interp)
export(clip_data)
export(compute_orientation)
export(custom_mims_unit)
export(cut_off_signal)
export(export_to_actilife)
export(extrapolate)
export(extrapolate_rate)
export(extrapolate_single_col)
export(generate_interactive_plot)
export(iir)
export(illustrate_extrapolation)
export(illustrate_signal)
export(import_actigraph_count_csv)
export(import_actigraph_csv)
export(import_actigraph_csv_chunked)
export(import_actigraph_meta)
export(import_activpal3_csv)
export(import_enmo_csv)
export(import_mhealth_csv)
export(import_mhealth_csv_chunked)
export(interpolate_signal)
export(mims_unit)
export(mims_unit_from_files)
export(parse_epoch_string)
export(sampling_rate)
export(segment_data)
export(sensor_orientations)
export(shiny_app)
export(simulate_new_data)
export(sum_up)
export(vector_magnitude)
importFrom(magrittr,"%>%")
